#ifndef LATTICEPROTEINPIVOTMOVESTATE_HH_
#define LATTICEPROTEINPIVOTMOVESTATE_HH_


#include "State.hh"
#include "LatticeProtein.hh"
#include "PivotMoveSet.hh"

namespace ell
{
		/**
		 * This class generates neighbors of lattice proteins in the
		 * energy landscape by applying pivot moves.
		 * 
		 * @author Martin Mann, Sebastian Will, Andreas Richter
		 */
	class LatticeProteinPivotMoveState : public State
	{
	protected:	
			/*! A pivot move on a lattice protein at position PivotMove.first
			 *  with move index  PivotMove.second.
		 	 */
		typedef std::pair<size_t, size_t> PivotMove;

			//! Constant that represents a invalid structure index position.
		static const size_t INVALID_INDEX;

		class NeighborList : public State::NeighborList
		{
		protected:
				//! the state of origin of the neighbors
			const LatticeProteinPivotMoveState* const origin;

		public:
				//! State of an iterator. Usually the information in ItState is
				//! used for computing the next list entry 
			class ItState : public State::NeighborList::ItState
			{
			public:
					//! the last applied pivot move
				PivotMove lastMove;

				ItState() : lastMove() {}
				ItState(const ItState &itstate)
				 : lastMove(itstate.lastMove) {}
				virtual ~ItState() {}

				virtual ItState* clone() const {
					return new ItState(*this);
				}
			};

			NeighborList(const LatticeProteinPivotMoveState* _origin);
			
			virtual ~NeighborList();

				/*! Returns a pointer to the first element of the
				 *  virtual list of neighbors of the state of origin.
				 *  Returns NULL if no neighbors exist.
				 */
			virtual State* first(State::NeighborList::ItState** itstate) const;
						 
				/*! Returns a pointer to the next element of the
				 *  virtual list of neighbors of the state of origin.
				 *  Returns NULL if no more neighbors exist.
				 */
			virtual State* next(	State::NeighborList::ItState* itstate, 
									State* elem) const;
		};

		class RandomNeighborList : public State::NeighborList
		{
		public:
			class ItState;
			
		protected:
				//! the state of origin of the neighbors
			const LatticeProteinPivotMoveState* const origin;
			
				//! Changes the mode of choice of the next pivot move from
				//! choice out of all pivot moves to choice out of all
				//! unchosen pivot moves
			void	switchMode(ItState* itstate) const;
			
		public:
			class ItState : public State::NeighborList::ItState
			{
			public:
					//! if the numbers of pivot moves that have been
					//! selected more than once exceeds this threshold, the
					//! mode of choice will be changed to choice out of all
					//! unchosen moves
				static const size_t SWITCH_MODE_THRESHOLD;
				
					//! counts down from SWITCH_MODE_THRESHOLD to zero for
					//! every move chosen more than ones
				size_t switchModeValue;
				
					//! set containing all pivot moves that were randomly
					//! chosen during iteration
				std::set<PivotMove> chosenMoves;
				
					//! vector containing all pivot moves which have not been
					//! chosen yet
				std::vector<PivotMove> unchosenMoves;

				ItState() : switchModeValue(SWITCH_MODE_THRESHOLD),
							chosenMoves(),
							unchosenMoves()
				{}
				
				ItState(const ItState &itstate) 
					: switchModeValue(itstate.switchModeValue),
					  chosenMoves(itstate.chosenMoves),
					  unchosenMoves(itstate.unchosenMoves)
				{}
				
				virtual ~ItState() 
				{}

				virtual ItState* clone() const {
					return new ItState(*this);
				}
			};

			RandomNeighborList(const LatticeProteinPivotMoveState* _origin);
			
			virtual ~RandomNeighborList();

				/*! Returns a pointer to a random element of the
				 *  virtual list of neighbors of the state of origin.
				 *  Returns NULL if no neighbors exist.
				 */
			virtual State* first(State::NeighborList::ItState** itstate) const;
						 
				/*! Returns a pointer to the next random element of the
				 *  virtual list of neighbors of the state of origin.
				 *  Returns NULL if no more neighbors exist.
				 */
			virtual State* next(	State::NeighborList::ItState* itstate, 
									State* elem) const;
		};

		LatticeProtein	latProt;

		PivotMoveSet	pivotMoveSet;

			/*! Returns the first pivot move which could be valid or invalid.
			 *  Returns PivotMove(0,INVALID_INDEX) if no first pivot move
			 *  exits.
			 */
		PivotMove	firstPivotMove() const;
		
			/*! Returns the next pivot move which could be valid or invalid
			 *  if lastPivotMove was the last pivot move.
			 *  Returns PivotMove(0,INVALID_INDEX) if no next pivot move
			 *  exits.
			 */
		PivotMove	nextPivotMove(const PivotMove lastPivotMove) const;

	public:
		LatticeProteinPivotMoveState(const LatticeModel* lattice, 
									 const ContactEnergyFunction* energy, 
									 const std::string& seq, 
									 const std::string& moveString, 
									 const bool isAbsoluteMove);
				
		LatticeProteinPivotMoveState( 
						const LatticeProteinPivotMoveState& latProtPMState2);
						
		virtual ~LatticeProteinPivotMoveState();

	// abstract functions (State)
		virtual bool	operator== (const State& state2) const;
		virtual bool	operator!= (const State& state2) const;

			/*! Returns the contact energy of this lattice protein state.
			 */
		virtual int	getFitness() const;
		
			/*! Returns the Hamming distance from this to another valid
			 *  State.
			 *  The distance is defined as the number of diferent positions
			 *  in the relative move string of the two states. Therefore,
			 *  the Hamming distance is the minimal number of pivot moves
			 *  to reach state2.
			 *  @param state2 the State to reach
			 */
		virtual unsigned int	getMinimalDistance(const State& state2) const;
		
			/*! Returns a pointer to a clone of the current state. */
		virtual LatticeProteinPivotMoveState*	clone() const;
		
			/*! Returns a specific std::string representation of this
			 *  State.
			 * 
			 *  ABSOLUTEMOVESTRING(SEQUENCE) */
		virtual std::string getStateStringRepresentation() const;

		virtual CState* compressedClone() const;

		virtual void	compress(CState& cState) const;

		virtual NeighborListPtr getNeighborList() const;
		
		virtual NeighborListPtr getRandomNeighborList() const;

	};

} // namespace ell

#endif /*LATTICEPROTEINPIVOTMOVESTATE_HH_*/
