#include <ell/LatticeModel.hh>

namespace ell
{
	
	LatticeModel::LatticeModel(const LatticeDescriptor* const _latDescriptor) 
		: latDescriptor(_latDescriptor)
	{
		assertell(latDescriptor != NULL, 
			"tried to create a LatticeModel without a LatticeDescriptor");
	}
	
	LatticeModel::LatticeModel(const LatticeModel& toCopy) :
		latDescriptor(toCopy.latDescriptor)	
		// latDescriptor wird nur kopiert, weil nicht vom LatticeModel 
		// wieder freizugegeben
	{
		assertell(latDescriptor != NULL, 
			"tried to create a LatticeModel without a LatticeDescriptor");
	}
	
	LatticeModel::~LatticeModel()
	{
	}
	
	bool
	LatticeModel::operator ==(const LatticeModel& lm2) const {
		return *latDescriptor == *(lm2.latDescriptor);
	}
	
	bool
	LatticeModel::operator !=(const LatticeModel& lm2) const {
		return *latDescriptor != *(lm2.latDescriptor);
	}
	
	// ------------------------------------------------------------
	// conversion of move sequences
	//
	
	MoveSequence
	LatticeModel::relMovesToAbsMoves( const MoveSequence &relMoves ) const {
		// get ref to neighborhood
		const LatticeNeighborhood &neighborhood = latDescriptor->getNeighborhood();
		
		MoveSequence absMoves; // here we store the result
		
		// base = matrix of first move
		Automorphism base=neighborhood.getElement(0).getRel2AbsRotation();
		
		for (unsigned int i=0; i<relMoves.size(); ++i) {
			
			base = base
				* neighborhood.getElement(relMoves[i]).getRel2AbsRotation();
				
			IntPoint absVec = base.columnVec(0); // the absolute move vector
			
			// get abs move corresponding to absVec
			absMoves.push_back(
				neighborhood.getElement(absVec).getMove()
				);
		}
		
		return absMoves;
	}
	
	MoveSequence
	LatticeModel::absMovesToRelMoves( const MoveSequence &absMoves ) const {
		// get ref to neighborhood
		const LatticeNeighborhood &neighborhood = latDescriptor->getNeighborhood();
		
		MoveSequence relMoves; // here we store the result
    
		// set base, s.t. first absolute move points "forward"
		Automorphism base 
			= neighborhood.getElement(absMoves[0]).getAbs2RelRotation();
    
		for (unsigned int i=0; i<absMoves.size(); ++i) {
			const Move &a = absMoves[i];
			
			// multiply Matrix by vector<int> (results in vector<int>) 
			const IntPoint relVec = base * neighborhood.getElement(a);
			
			// get move with vector relVec
			Move relMove = neighborhood.getElement(relVec).getMove();
			relMoves.push_back(relMove);
			
			base = neighborhood.getElement(relMove).getAbs2RelRotation()
				* base;
			
		}
		return relMoves;
	}

	//
	// ------------------------------------------------------------

	
	// ------------------------------------------------------------
	// conversion moves <=> coordinates
	//

	IntPoint 
	LatticeModel::applyAbsMove( const IntPoint& actPoint, 
								const Move& absMove) const {
		assertell(latDescriptor != NULL, 
			"LatticeModel has no LatticeDescriptor");
		
		LatticeNeighborhood::const_iterator neighbor;
		LatticeNeighborhood::const_iterator end = 
			latDescriptor->getNeighborhood().end();
		for (neighbor = latDescriptor->getNeighborhood().begin(); 
			 neighbor != end; neighbor++)
			if (neighbor->getMove() == absMove) {
				return actPoint + (*neighbor);
			}
		// fehlerfall .. absmove nicht gefunden!
		assertell(neighbor != end,
				  "cant find the absolute move in the neighborhood"
				  "of the LatticeDescriptor");
		return IntPoint(actPoint);
	}
	
	
	IPointVec 
	LatticeModel::absMovesToPoints( const MoveSequence& absMoveSeq ) const {
		assertell(latDescriptor != NULL, 
			"LatticeModel has no LatticeDescriptor");
		
		IPointVec points;	// rueckgabe vector
		points.push_back(ell::IntPoint(0,0,0)); // init mit lattice center
		for (MoveSequence::size_type i=0; i< absMoveSeq.size(); i++){
				// berechne naechsten auf basis des letzten punktes
			points.push_back( applyAbsMove( points[i], absMoveSeq[i] ) );	
		}
		return points;
	}
	
	IPointVec 
	LatticeModel::relMovesToPoints( const MoveSequence& relMoves) const {
		assertell(latDescriptor != NULL, 
			"LatticeModel has no LatticeDescriptor");
		
		MoveSequence absMoves = relMovesToAbsMoves(relMoves);
		return absMovesToPoints(absMoves);
	}
	
	Move 
	LatticeModel::getAbsMove(	const IntPoint& lastPoint, 
								const IntPoint& actPoint) const {
		assertell(latDescriptor != NULL, 
			"LatticeModel has no LatticeDescriptor");
		return  latDescriptor->getNeighborhood().getElement(
					actPoint-lastPoint).getMove();		// vektor holen
	}
	
	
	MoveSequence 
	LatticeModel::pointsToAbsMoves( const IPointVec& points ) const {
		MoveSequence absmoves;	 // erstelle vector der entsprechenden groesse
		ell::IPointVec::const_iterator act = points.begin(), last = act;
		for (act++; act != points.end(); last = act++) {
				// haenge nacheinander moves an
			absmoves.push_back(getAbsMove(*last, *act));	
		}
		return absmoves;
	}
	
	MoveSequence 
	LatticeModel::parseMoveString( const std::string& moveStr) const {
		assertell(latDescriptor != NULL, 
			"LatticeModel has no LatticeDescriptor");
		return latDescriptor->getSequence(moveStr);
	}

	MoveSequence 
	LatticeModel::pointsToRelMoves( const IPointVec& points) const {
		assertell(latDescriptor != NULL, 
			"LatticeModel has no LatticeDescriptor");
		return absMovesToRelMoves(pointsToAbsMoves(points));	
	}
		
	
	std::string	
	LatticeModel::getString(const MoveSequence& moveSeq) const {
		assertell(latDescriptor != NULL, 
			"LatticeModel has no LatticeDescriptor");
		return latDescriptor->getString(moveSeq);
	}
	
	
	bool 
	LatticeModel::areNeighbored(	const IntPoint& first, 
									const IntPoint& second) const {
		assertell(latDescriptor != NULL, 
			"LatticeModel has no LatticeDescriptor");
		return latDescriptor->getNeighborhood().isElement(second-first);
	}
	
	IPointSet 
	LatticeModel::getAllNeighPoints(const IntPoint& center) const
	{ 
		assertell(latDescriptor != NULL, 
			"LatticeModel has no LatticeDescriptor");
		LatticeNeighborhood neighborhood = latDescriptor->getNeighborhood(); 
		IPointSet retSet;
		for (LatticeNeighborhood::const_iterator it = neighborhood.begin(); 
					it != neighborhood.end(); it++) {
			retSet.insert(center + (*it));
		}
		return retSet;
	}

} // namespace ell
