#include <ell/RNAFreeEnergy.hh>
#include <ell/assertell.hh>

extern "C"	float energy_of_struct(const char *string, const char *structure);

namespace ell
{

	const Alphabet RNAFreeEnergy::COMMON_ALPHABET("ACGU", 1);
	const AllowedBasePairs RNAFreeEnergy::COMMON_BPAIRS(
							&COMMON_ALPHABET, "AU,CG,GU");
//////////////////////////////////////////////////////////
// RNAFreeEnergy
//////////////////////////////////////////////////////////

	RNAFreeEnergy::RNAFreeEnergy(const std::string& rnaSeqStr,
	 							 const std::string& rnaStructBracketNotStr)
	 :	RNAStructure(rnaSeqStr, rnaStructBracketNotStr, &COMMON_BPAIRS) 
	{
	}
	
	RNAFreeEnergy::RNAFreeEnergy(const RNAFreeEnergy& rnaFreeEnerg)
	 : RNAStructure(rnaFreeEnerg) 
	{
	}

	RNAFreeEnergy::~RNAFreeEnergy()	{
	}

	size_t	
	RNAFreeEnergy::nextCompatibleSingleMovePos( size_t pos) const {
		if (pos+1 < rnaStructBracketNot->size()) {
			if ((*rnaStructBracketNot)[pos+1] == STRUCT_ALPH.getElement(".")) {
				return pos+1;
			}
			else if ((*rnaStructBracketNot)[pos+1] ==
					   STRUCT_ALPH.getElement("(")) {
					return nextCompatibleSingleMovePos(rnaBonds[pos+1]);
			}
			else {
					return RNAStructure::INVALID_INDEX;
			}
		}
		else {
			return RNAStructure::INVALID_INDEX;
		}
	}

	void		
	RNAFreeEnergy::applySingleMoveInPlace(const SingleMove& move) {
		// open bond between move.first and move.second
		if (rnaBonds[move.first] == move.second) {
			(*rnaStructBracketNot)[move.first] = STRUCT_ALPH.getElement(".");
			(*rnaStructBracketNot)[move.second] = STRUCT_ALPH.getElement(".");
			rnaBonds[move.first] = RNAStructure::INVALID_INDEX;
		}
		// close bond between pos1 and pos2
		else {
			assertell(isValidSingleMove(move),
				"can't close bond with single move, because it is not valid");
			(*rnaStructBracketNot)[move.first] = STRUCT_ALPH.getElement("(");
			(*rnaStructBracketNot)[move.second] = STRUCT_ALPH.getElement(")");
			rnaBonds[move.first] = move.second;
		}
	}

	bool		
	RNAFreeEnergy::isValidSingleMove(const SingleMove& move)
			 const {
		if (move.first >= move.second
		    || move.second >= rnaStructBracketNot->size()) {
			return false;
		}

		// bond between move.first and move.second
		if (rnaBonds[move.first] == move.second) {
			return true;
		}
		// no bond between move.first and move.second
		// move.first or move.second are paired, but not with each other
		if ( !((*rnaStructBracketNot)[move.first] ==
				STRUCT_ALPH.getElement(".") 
			 && (*rnaStructBracketNot)[move.second] ==
				 STRUCT_ALPH.getElement("."))) {
			return false;
		}

		// both move.first and move.second are unpaired
		// distance between move.first and move.second has to be at least
		//  MIN_LOOP_LENGTH
		if ((move.second-move.first) <= MIN_LOOP_LENGTH) {
			return false;
		}
		// iterate all positions which preserve the nested property
		// starting after move.first to determine if move.second is one
		// of this positions
		for (size_t i = nextCompatibleSingleMovePos(move.first);
		     i < rnaStructBracketNot->size() && i <= move.second;
		     i = nextCompatibleSingleMovePos(i)) {
			if (move.second == i) {
				return isAllowedBasePair(move.first, move.second);
			}
		}
		// bond between move.first and move.second would
		// destroy the nested property
		return false;
	}
	
	double 		
	RNAFreeEnergy::getEnergy() const {
		return energy_of_struct(
			(bPair->getAlphabet()->getString(*rnaSeq)).c_str(),
			(STRUCT_ALPH.getString(*rnaStructBracketNot)).c_str() );
	}

} // namespace ell
